#include <BoolField.h>
#include <DegreeField.h>
#include <Field.h>
#include <Form.h>
#include <IntField.h>
#include <LCD.h>
#include <ListField.h>
#include <TextField.h>
#include <TimeField.h>
#include <GetIntField.h>
#include <GetMicroField.h>
#include <TimerOne.h>
#include <EEPROM.h>
#include <avr/wdt.h>
#include <avr/pgmspace.h>
#include "EEPROMAnything.h"

void goDivide(long gAngle);
void goJog(void);
void enterDegrees(void);
void enterDivisions(void);
void goSetSpeed(void);
void goHome(void);
void goSetJogStep(void);
void setSpeed(int spdPct);
void goConTurn(void);
void doTravel(long dist);
void enterDistance(void);
long HzToTimeDelay(long iHz);
long TimeDelayToHz(long tDelay);
int getGlobals(int gDevice);
int putGlobals(void);
void doSetupMenu(void);
void setSteps(long sSteps, int sWait);  // If sWait is true, it will wait until steps are complete
void rampDown(void);
void doConfigMenu(void);
void eraseEeprom(char *sPrompt);
void addDevice(char *pPrompt);
void setdeviceType(int dType);
void printDistanceVals(long DistToTravel, int step);

void doTravel(long idistance);


#define STEP_PIN             1
#define DIR_PIN              2
#define MINUTES_IN_CIRCLE    1296000      // Number od seconds in a 360 degree circle
#define MAX_LONG             2147483646   // The biggest number we can use with a long
#define CONFIG_VERSION       "RT4"        // The Sigature to tell us we've saved stuff in the EEPROM
#define NO_DEVICE_SIG        "XXX"        // The Sigature to tell us we've got nothing in the EEPROM
#define DEV_ROTARY           0            // Device Type - Rotary Table
#define DEV_LINEAR           1            // Device Type - Linear device mm

#define SUB_VER              1            // Change on each version
#define STEPPER_STEPS_REV    200          // The number of steps per stepper
#define MICRO_STEPS          10           // set to 1 if not not using a micostepper driver, otherwise set to # microsteps
#define DIVIDE_RATIO         90           // The Dividing table ratio (eg 40 or 90)
#define MM_PER_REV           2            // Travel per mm
#define MAX_STEPPER_HZ       10000        // Maximum frequency for this stepper
#define MIN_STEPPER_HZ       5000         // Minimum frequency for this stepper
#define TIMER_DELAY          80           // 80 MIcroseconds between interrupt ticks
#define JOG_STEP_ANGLE       1800L        // 0.5 degrees (30 seconds)

struct globals_t  // Size = 40 bytes
{

  int  devType;
  int  DirPin;
  int  StepPin;
  long stepperStepsPerRev;
  int  microSteps;
  int  divideRatio;
  long maxStepperHz;
  long minStepperHz;
  long stepsPerRev;
  long JogStepAngle;
  long TravelDistRev;
  long JogStepCount;
  int  dir;
  long timerDelay;
} globals={
    DEV_ROTARY,
    DIR_PIN,
    STEP_PIN,
    (long)STEPPER_STEPS_REV,
    MICRO_STEPS,
    DIVIDE_RATIO,
    (long)MAX_STEPPER_HZ,
    (long)MIN_STEPPER_HZ,
    ((long)STEPPER_STEPS_REV * (long)MICRO_STEPS * (long)DIVIDE_RATIO),    
    (long)JOG_STEP_ANGLE,
    2000L,  // 
    (long)(((float)STEPPER_STEPS_REV * (float)MICRO_STEPS * (float)DIVIDE_RATIO) *((float)JOG_STEP_ANGLE/(float)MINUTES_IN_CIRCLE)),
    0, 
    TIMER_DELAY
};

struct config_t // Size = 10 bytes
{
  char version[4];
  int subVer;    // Change this each release 
  int currDevice;
  int numDevices;
}configuration;

LCD lcd;
int CLOCKWISE    =           0;
int ANTICLOCKWISE =          1;
volatile int state = CLOCKWISE;
volatile long steps = 0L;      // Number of steps to process
volatile long stepcount = 0;  // Number of steps done;
long rampSteps;
volatile int stepmod=0;

volatile long endRampUp;       // Steps to stop ramping up
volatile long startRampDown;   // Steps to start ramping down
static long startDelay;        // Start delay - slow speed stepper start pulse delay in microseconds 
static long targetDelay;       // Target delay - Full speed stepper pulse delay in microseconds
static long sDelay;            // Current delay for stepper movement

volatile long homePos = 0L;    // Remember the home position
long divisions=3;
long distance =10000L;
long stepsPerDiv=0;
long mAngle = 0L;
int goinghome = 0;  // True if Table is going home
long Operating_Hz;  // The frequency the steper is operating at

const char item_Start[] PROGMEM = "UP/DWN Starts";
 ListItem const TopMenu[] PROGMEM = {
    item_Start,
    0
 };
const char yStr[] = "Yes";
const char nStr[] = "No";
const char item_DeviceRot[] PROGMEM = "Rotary";
const char item_DeviceMM[] PROGMEM = "Linear";

  ListItem const iDevType[] PROGMEM = {
    item_DeviceRot,
    item_DeviceMM,
    0
  };
 Form mainForm(lcd);
 ListField goDivideField(mainForm, "<Divide #>" , TopMenu);
 ListField goAngleField(mainForm, "<Div Angle>" , TopMenu);
 ListField goTravelField_mm(mainForm, "<Travel>" , TopMenu);
 ListField goJogField(mainForm,"<Jog>" , TopMenu);
 ListField goHomeField(mainForm, "<Home>" ,TopMenu);
 ListField goConTurnField(mainForm,"<Contin>",TopMenu);
 ListField goConfigField(mainForm,"<Config>" ,TopMenu);


void setup() 
{
   const char welcome1[] PROGMEM = "VMN Ctrl";
   const char welcome2[] PROGMEM = "V:";
   const char welcome3[] PROGMEM = "D=";
   const char rotStr[] PROGMEM = "Rot";
   const char linStr[] PROGMEM = "Lin";

   long previousMillis = 0;
   unsigned long currentMillis;
   long interval = 2000L;
   int event = LCD_BUTTON_NONE;

  MCUSR = 0;  // clear out any flags of prior resets so we can reboot
  //read the config from EEPROM
  startDelay =   HzToTimeDelay(globals.minStepperHz);          
  targetDelay =  HzToTimeDelay(globals.maxStepperHz);  
  sDelay = targetDelay;
  EEPROM_readAnything(0, configuration);
   {
     const char sig[] PROGMEM = CONFIG_VERSION;
     int ok = 1;
     int cs = sizeof(configuration);
     char *s = (char *)sig;
     char *p= (char *)&configuration;
     while(*s){
       if(*s != *p){
         ok = 0;
         break;
       }
       s++; p++; 
     }
     if(!ok){     
       // No data in EEprom, so set the data
       strcpy(configuration.version,sig);
       configuration.subVer = SUB_VER;
       configuration.currDevice = 0;
       configuration.numDevices = 0;  //First device in C++ = 0
       EEPROM_writeAnything(0, configuration); // write to EEProm
       globals.timerDelay = sDelay;
       EEPROM_writeAnything(cs,globals); // Write the defaults to EEprom starting at the first byte after configuration
     }
     else{
       getGlobals(configuration.currDevice);
     }
   }
   // We've read everything from EEPROM, so set the defaults   
   mAngle =(long)((float)MINUTES_IN_CIRCLE / (float)divisions);
   float tmpSteps = (float) globals.stepsPerRev;  // (float)STEPS_PER_REV
   stepsPerDiv = (long)(tmpSteps *((float)mAngle/(float)MINUTES_IN_CIRCLE));
   globals.JogStepCount  = (long)(tmpSteps *((float)globals.JogStepAngle/(float)MINUTES_IN_CIRCLE)) * 2L;
   startDelay =   HzToTimeDelay(globals.minStepperHz);          
   targetDelay =  HzToTimeDelay(globals.maxStepperHz);  
   sDelay = targetDelay;
   if(globals.dir){
     CLOCKWISE =      1;
     ANTICLOCKWISE =  0;
   }
   else{
     CLOCKWISE =      0;
     ANTICLOCKWISE =  1;
   }
   pinMode(globals.StepPin, OUTPUT);
   pinMode(globals.DirPin, OUTPUT);
   Timer1.initialize(sDelay); // set a timer of length in microseconds (or 0.1 sec - or 10Hz)
   Timer1.attachInterrupt( timerIsr ); // attach the service routine here
   // Enable the screen saver, which will automatically blank the screen after 30 seconds. You may wish to comment this line out if it annoys you
   lcd.enableScreenSaver(30);
   Operating_Hz = globals.maxStepperHz;
   //globals.stepsPerRev = 180000L;   // RJW Check deleting this does not break stuff, shouldn't be hardocding stuff, probably from debugging
   setdeviceType(globals.devType);
   // Print a welcome message  
   lcd.clear();
   lcd.setCursor(0,0);   
   lcd.print(welcome1);
   lcd.setCursor(0,1);
   lcd.print(welcome2); 
   lcd.print(configuration.version);
   lcd.print(".");
   lcd.print(configuration.subVer);
   lcd.setCursor(8,1);
   lcd.print(welcome3); 
   lcd.print(configuration.currDevice);
   lcd.setCursor(13,1);
   if(globals.devType == DEV_ROTARY){
     lcd.print(rotStr);
   }
   else{
     lcd.print(linStr);
     globals.stepsPerRev = (long)((float)globals.stepperStepsPerRev * (float)globals.microSteps * ((float)globals.divideRatio/1000.0)); // Calculate linear stes per rev
   }
   while(1){  // Delay 2 seconds without using delay();
     currentMillis = millis();
     if(currentMillis - previousMillis > interval) {
        previousMillis = currentMillis;
        break;  
     }
     event = lcd.getButton();
   }
   lcd.clear();
 
  // Show the main form for the first time.
  mainForm.show();
}
 
void loop()
{
  int i;

  // Dispatch button events to the main form.
  int event = lcd.getButton();
  if (mainForm.dispatch(event) == FORM_CHANGED) {
    if (mainForm.isCurrent(goDivideField)) {
      enterDivisions();
      mainForm.setCurrentField(&goDivideField);
    }
    if (mainForm.isCurrent(goJogField)) {
      goJog();
      mainForm.setCurrentField(&goJogField);
    }
    if (mainForm.isCurrent(goAngleField)) {
      enterDegrees();
      mainForm.setCurrentField(&goAngleField);
    }
    if (mainForm.isCurrent(goHomeField)) {
      goHome();
      mainForm.setCurrentField(&goHomeField);
    }
    if (mainForm.isCurrent(goConTurnField)) {
      goConTurn();
      mainForm.setCurrentField(&goConTurnField);
    }
    if (mainForm.isCurrent(goConfigField)) {
      doConfigMenu();
      mainForm.setCurrentField(&goConfigField);
    }
    if (mainForm.isCurrent(goTravelField_mm)) {
      enterDistance();
      mainForm.setCurrentField(&goTravelField_mm);
    }

  }
}
 
void printDivideVals(int iDiv,long iAngle)
{
  Form tempForm(lcd);
  DegreeField tempField(tempForm, "", 360, DEGREEFIELD_READ_WRITE); // Just using this to convert to degrees

  lcd.clear();
   lcd.setCursor(0,0);
   lcd.print("Div: ");
   lcd.print(iDiv);
   lcd.print(" of ");
   lcd.print(divisions);
   tempField.setValue(iAngle);
   tempField.printDegrees();
   lcd.setCursor(10,1);
   lcd.print(" / Div"); 
  tempForm.removeField(&tempField); 
}


void goDivide(long gAngle)
{
   long thisDiv = 1; 
   int event = LCD_BUTTON_NONE;
   float tmpSTEPS_PER_REV = (float)globals.stepsPerRev;
   
   stepsPerDiv = (long)(tmpSTEPS_PER_REV *((float)gAngle/(float)MINUTES_IN_CIRCLE));
   divisions = (long)((float) MINUTES_IN_CIRCLE / (float) gAngle);
   printDivideVals(thisDiv,gAngle);
   while(event != LCD_BUTTON_SELECT){
      event = lcd.getButton();
      switch(event){
        case LCD_BUTTON_LEFT:
          thisDiv--;
          if(thisDiv < 1)
            thisDiv = divisions;          
          digitalWrite( globals.DirPin, ANTICLOCKWISE );
          setSteps(stepsPerDiv * 2, 1);
          printDivideVals(thisDiv,gAngle);
          break;
        case LCD_BUTTON_RIGHT:
          thisDiv++;
          if(thisDiv > divisions)
            thisDiv = 1;          
          digitalWrite( globals.DirPin, CLOCKWISE );
          setSteps(stepsPerDiv * 2,1); 
          printDivideVals(thisDiv, gAngle);
          break;
    } 
  }
}

void goJog(void)
{
   int event,event2 = LCD_BUTTON_NONE;
   const char jog1[] PROGMEM = "CONT: Left/Right";
   const char jog2[] PROGMEM = "STEP: Up/Down";

   lcd.clear();
   lcd.setCursor(0,0);
   lcd.print(jog1);
   lcd.setCursor(0,1);
   lcd.print(jog2); 
   
   while(event != LCD_BUTTON_SELECT){
      event = lcd.getButton();
      if(sDelay < HzToTimeDelay(globals.maxStepperHz))
        sDelay = HzToTimeDelay(globals.maxStepperHz);
      switch(event){
        case LCD_BUTTON_LEFT:
          digitalWrite( globals.DirPin, ANTICLOCKWISE );
          setSteps(MAX_LONG, 0);
          break;
        case LCD_BUTTON_RIGHT:
          digitalWrite( globals.DirPin, CLOCKWISE );
          setSteps(MAX_LONG,0);  
          break;
        case LCD_BUTTON_UP:
          digitalWrite( globals.DirPin, CLOCKWISE );
          setSteps(globals.JogStepCount, 1);
          break;
        case LCD_BUTTON_DOWN:
          digitalWrite( globals.DirPin, ANTICLOCKWISE );
          setSteps(globals.JogStepCount, 1);
          break;
        case LCD_BUTTON_LEFT_RELEASED:
        case LCD_BUTTON_RIGHT_RELEASED:
          steps = 0;       
          rampDown();    // Need to add enough steps to slow down
          break;
       }
    } 
    steps = 0L;
}

void enterDegrees(void)
{
  
  Form angleForm(lcd);
  DegreeField angleField(angleForm,"Set Ang" , 360, DEGREEFIELD_READ_WRITE);
  
  int event = LCD_BUTTON_NONE;

  angleField.setValue(mAngle);
  angleForm.defaultField();
  angleForm.show();
  while(event!= LCD_BUTTON_SELECT){
    event = lcd.getButton();
    if(event == LCD_BUTTON_SELECT)
      break;
    if (angleForm.dispatch(event) == FORM_CHANGED) {
        if (angleForm.isCurrent(angleField)) {
          mAngle = angleField.value();
          float tmpSteps = (float) globals.stepsPerRev;  // (float)STEPS_PER_REV
          divisions = (long)(tmpSteps *((float)mAngle/(float)MINUTES_IN_CIRCLE));
        }
    }
  }
  if(mAngle)
    goDivide(mAngle);
}

void enterDivisions(void)
{
  Form divideForm(lcd);
  GetIntField divfField(divideForm,"Divs" , 9999, GETINTFIELD_READ_WRITE);
  
  int event = LCD_BUTTON_NONE;
  divfField.setValue(divisions);
  divideForm.defaultField();
  divideForm.show();
  while(event != LCD_BUTTON_SELECT){
    event = lcd.getButton();
    if (divideForm.dispatch(event) == FORM_CHANGED) {
        if (divideForm.isCurrent(divfField)) {
          divisions = divfField.value();        
          mAngle=(long)((float)MINUTES_IN_CIRCLE / (float)divisions);
       }
    }
  }
  if(divisions)
    goDivide(mAngle);
  divideForm.removeField(&divfField); 
}

void goSetSpeed(void)
{
  Form speedForm(lcd);
  IntField speedField(speedForm,"Spd" , 0, 100, 5, 100, "%");
  int speedPct = (int)((((float)((float)globals.maxStepperHz - (float)globals.minStepperHz) / (float)(Operating_Hz - globals.minStepperHz)))*100.0);
  int event = LCD_BUTTON_NONE;
  lcd.clear();
  speedField.setValue(speedPct);
  speedForm.defaultField();
  speedForm.show();
  while(event != LCD_BUTTON_SELECT ){
    event = lcd.getButton();
    if (speedForm.dispatch(event) == FORM_CHANGED) {
        if (speedForm.isCurrent(speedField)) {
          speedPct = speedField.value();
        }
    }
  }
  setSpeed(speedPct);
}

void setSpeed(int spdPct)
{
  globals.maxStepperHz =  (long)((float)( Operating_Hz - globals.minStepperHz) * (float)spdPct/100.0) + globals.minStepperHz;
  globals.timerDelay = HzToTimeDelay(globals.maxStepperHz);
  Timer1.setPeriod(globals.timerDelay);      // set a timer of length in microseconds
}

void goHome(void)
{
   int event = LCD_BUTTON_NONE;
   long tmpSteps = (long)globals.stepsPerRev;
   
   const char home1[] PROGMEM = "SET:  Left/Right";
   const char home2[] PROGMEM = "GOTO: Up/Down";
   const char home3[] PROGMEM = "HOME Set  ";
   const char home4[] PROGMEM = "Going HOME";
 
   lcd.clear();
   lcd.setCursor(0,0);
   lcd.print(home1);
   lcd.setCursor(0,1);
   lcd.print(home2); 

   while(1){
      event = lcd.getButton();
      if(event == LCD_BUTTON_SELECT)
        break;
      switch(event){
        //set home
        case LCD_BUTTON_LEFT:
        case LCD_BUTTON_RIGHT:
          lcd.setCursor(6,0);
          lcd.print(home3);
          while(steps){
            ;
          }
          homePos = 0;
          break;
        case LCD_BUTTON_UP:
        case LCD_BUTTON_DOWN:
        // Goto home
          lcd.setCursor(6,1);
          lcd.print(home4);
          goinghome++;
          if(globals.devType == DEV_ROTARY){  // Rotary Device, do the original code
            if(homePos <= (tmpSteps / 2L)){
              digitalWrite( globals.DirPin, ANTICLOCKWISE );
              setSteps(homePos * 2,1);
            }
            else{    
              digitalWrite( globals.DirPin, CLOCKWISE );
              setSteps((tmpSteps - homePos) * 2, 1);
            }
          }
          else{ // Linear device, check if we re to the left or right of home and set the direction     
            if(homePos > 0)
              digitalWrite( globals.DirPin, ANTICLOCKWISE );
            else
              digitalWrite( globals.DirPin, CLOCKWISE );          
            setSteps(abs(homePos) * 2,1);
          }
          while(steps){
            ;
          }
          homePos = 0;
          goinghome = 0;
          break;
       default:
         break;
      }
   }
}

void goSetJogStep(void)
{
  Form jogStepForm(lcd);
  DegreeField jogStepField(jogStepForm,"Jog Angle" , 10, DEGREEFIELD_READ_WRITE);

  int event = LCD_BUTTON_NONE;
  float tmpSteps = (float) globals.stepsPerRev;  // (float)STEPS_PER_REV

  lcd.clear();
  jogStepField.setValue(globals.JogStepAngle);
  jogStepForm.defaultField();
  jogStepForm.show();
  while(event != LCD_BUTTON_SELECT ){
    event = lcd.getButton();
    if (jogStepForm.dispatch(event) == FORM_CHANGED) {
        if (jogStepForm.isCurrent(jogStepField)) {
          globals.JogStepAngle = jogStepField.value() * 2;
          globals.JogStepCount = (long)(tmpSteps *((float)globals.JogStepAngle/(float)MINUTES_IN_CIRCLE));  
        }
    }
  }
}


void goConTurn(void)
{

  Form conTurnForm(lcd);
  TextField conTurnField(conTurnForm,"Cont", "SEL Stop" );
  int direction = digitalRead(globals.DirPin);
  int event = LCD_BUTTON_NONE;
  lcd.clear();
  conTurnForm.defaultField();
  conTurnForm.show();
  rampUp();
  steps = MAX_LONG;
  while(event != LCD_BUTTON_SELECT ){
    if(sDelay < HzToTimeDelay(globals.maxStepperHz ))
      sDelay = HzToTimeDelay(globals.maxStepperHz );
    event = lcd.getButton();
    switch(event){
      case LCD_BUTTON_LEFT:
        if( direction != ANTICLOCKWISE){
          rampDown();
          while(steps){
            ;
          }
          direction = ANTICLOCKWISE;
          digitalWrite(globals.DirPin, direction );
          rampUp();
          steps = MAX_LONG;
        }     
        break;
      case LCD_BUTTON_RIGHT:
        if(direction != CLOCKWISE){
          rampDown();
          while(steps){
            ;
          }
          direction = CLOCKWISE;
           digitalWrite( globals.DirPin, CLOCKWISE );
          rampUp();
          steps = MAX_LONG;
        }       
        break;
    }      
      if (conTurnForm.dispatch(event) == FORM_CHANGED) {
        if (conTurnForm.isCurrent(conTurnField)) {
        }
    }
  }
  rampDown();
  while(steps){
    ;
  }
}

int getGlobals(int gDevice)
{
   // Get the configuraton data
   EEPROM_readAnything(gDevice * sizeof(globals) + sizeof(configuration),globals);  // configuraaton exists, so read the curent setup.
   // DO any further setup for this device
   setdeviceType(globals.devType);
   if(globals.dir){
     CLOCKWISE =      1;
     ANTICLOCKWISE =  0;
   }
   else{
     CLOCKWISE =      0;
     ANTICLOCKWISE =  1;
   }
   return 0;
}

int putGlobals(void)
{
  int offset = configuration.currDevice * sizeof(globals) + sizeof(configuration);  // calculate the offset based on setup number
  EEPROM_writeAnything(offset,globals); // Write the defaults to EEprom starting at the first byte after configuration
 
  return 0;
}

void doSetupMenu(void)
{
 
  Form setupForm(lcd);
  int saveSetup = false;
  ListField sTypeField(setupForm, "DevTyp",iDevType);
  GetMicroField sJogStepDistField(setupForm,  "JgMM", 999999, GETINTFIELD_READ_WRITE);
  DegreeField sJogStepAngleField(setupForm, "JgAn",360, DEGREEFIELD_READ_WRITE);
  IntField sDirPinField (setupForm, "DirP",0,20,1,2," ");
  IntField sStepPinField (setupForm,"StpP",0,20,1,1," ");
  GetIntField sStepperStepsRevField(setupForm,"Stp/Rv" , 9999, GETINTFIELD_READ_WRITE);
  IntField sMicroStepsField (setupForm,"M/Stp",0,99,1,10,":1");
  IntField sDivideRatioField (setupForm,"D/Rat",0,9999,1,globals.divideRatio,":1");
  GetMicroField sTravelRatioField (setupForm,"T/Rat",999999, GETINTFIELD_READ_WRITE);
  GetMicroField sTravelMMField (setupForm,"Dis/Rv",999999, GETINTFIELD_READ_WRITE);
  GetIntField sMaxStepperHzField(setupForm, "MaxHz", 999999, GETINTFIELD_READ_WRITE);
  GetIntField sMinStepperHzField(setupForm, "MinHz", 999999, GETINTFIELD_READ_WRITE);
  BoolField sDirField(setupForm, "SwapD?", yStr,nStr, false);
  BoolField sSaveField(setupForm, "Save?", yStr, nStr, false);

  int event = LCD_BUTTON_NONE;
  sDirPinField.setValue(globals.DirPin);
  sStepPinField.setValue(globals.StepPin);
  sStepperStepsRevField.setValue(globals.stepperStepsPerRev);
  sTravelMMField.setValue(globals.TravelDistRev);
  sMicroStepsField.setValue(globals.microSteps);
  sDivideRatioField.setValue(globals.divideRatio);
  sMaxStepperHzField.setValue(globals.maxStepperHz);
  sMinStepperHzField.setValue(globals.minStepperHz);
  sJogStepAngleField.setValue(globals.JogStepAngle);
  sJogStepDistField.setValue(globals.JogStepAngle);  
  //sDirField.setValue(globals.dir);
  sTypeField.setValue(globals.devType);
  switch(globals.devType){
  case DEV_ROTARY:
     setupForm.removeField(&sTravelMMField);
     setupForm.removeField(&sJogStepDistField);
     setupForm.removeField(&sTravelRatioField);
     break;  
  case DEV_LINEAR:
     setupForm.removeField(&sDivideRatioField);
     setupForm.removeField(&sJogStepAngleField);
     break;  
  }

  lcd.clear(); 
  setupForm.defaultField();
  setupForm.show();
  while(event != LCD_BUTTON_SELECT){
    event = lcd.getButton();
    if (setupForm.dispatch(event) == FORM_CHANGED) {      
      if (setupForm.isCurrent(sDirPinField)) {
          globals.StepPin = sStepPinField.value();
        }
       if (setupForm.isCurrent(sDirPinField)) {
          globals.DirPin = sDirPinField.value();
        }
        if (setupForm.isCurrent(sMicroStepsField)) {
          globals.microSteps = sMicroStepsField.value();
          globals.stepsPerRev = globals.stepperStepsPerRev * globals.microSteps * globals.divideRatio;
        }
        if (setupForm.isCurrent(sStepperStepsRevField)) {
          globals.stepperStepsPerRev = sStepperStepsRevField.value();
          globals.stepsPerRev = globals.stepperStepsPerRev * globals.microSteps * globals.divideRatio;
        }
        if (setupForm.isCurrent(sDivideRatioField)) {
          globals.divideRatio = sDivideRatioField.value();
          globals.stepsPerRev = globals.stepperStepsPerRev * globals.microSteps * globals.divideRatio;
        }
        if (setupForm.isCurrent(sTravelRatioField)) {
          globals.divideRatio = sTravelRatioField.value();
          globals.stepsPerRev = (long)((float)globals.stepperStepsPerRev * (float)globals.microSteps * ((float)globals.divideRatio/1000.0));
        }

        if (setupForm.isCurrent(sMaxStepperHzField)) {
          globals.maxStepperHz = sMaxStepperHzField.value();
        }
        if (setupForm.isCurrent(sMinStepperHzField)) {
          globals.minStepperHz = sMinStepperHzField.value();
        }
        if (setupForm.isCurrent(sJogStepAngleField)) {
          globals.JogStepAngle = sJogStepAngleField.value();
          float tmpSteps = (float) globals.stepsPerRev;  
          globals.JogStepCount  = (long)(tmpSteps *((float)globals.JogStepAngle/(float)MINUTES_IN_CIRCLE)) * 2L;       
        }
        if (setupForm.isCurrent(sDirField)) {
          if(sDirField.value()){
            globals.dir ^= globals.dir;
            if(globals.dir){
              CLOCKWISE =      1;
              ANTICLOCKWISE =  0;
            }
            else{
              CLOCKWISE =      0;
              ANTICLOCKWISE =  1;
            }
          }            \
        }
        if (setupForm.isCurrent(sTypeField)) {
           globals.devType = sTypeField.value();
           setdeviceType(globals.devType);
        }
        if (setupForm.isCurrent(sSaveField)) {
           saveSetup = sSaveField.value();         
        }        
    }   
  }    
  if(saveSetup){
    int retval = putGlobals();
    wdt_enable(WDTO_15MS);                          // REBOOT - turn on the WatchDog and don't stroke it.
    for(;;) { 
      // do nothing and wait for the eventual... REBOOT
    }
  } 
}


long HzToTimeDelay(long iHz)
{
  long retval = 0L;
  retval = (long)((1000000.0 /(float)iHz) / 2.0);
  return(retval);
}

long TimeDelayToHz(long tDelay)
{
  long retval = 0L;
  retval = (long)(((1000000.0 /(float)tDelay)) / 2.0);
  return(retval);
}

void rampDown(void)
{
  startDelay =   HzToTimeDelay(globals.minStepperHz);          
  targetDelay =  HzToTimeDelay(globals.maxStepperHz); 
  rampSteps = ((startDelay-targetDelay) * globals.microSteps);  
  startRampDown = rampSteps - 3L;      // Steps to start ramping down - steps count down so we start at a high #
  endRampUp =  rampSteps -1L;                      // We don't want to Ramp Up as we are supposed to be moving
  sDelay = targetDelay;
  steps = rampSteps + 1L;              // Run enough steps to stop gracefully
}

void rampUp(void)
{
  startDelay =   HzToTimeDelay(globals.minStepperHz);          
  targetDelay =  HzToTimeDelay(globals.maxStepperHz); 
  rampSteps = ((startDelay-targetDelay) * globals.microSteps);  
  startRampDown = 101L;      // Steps to start ramping down - steps count down so we start at a high #
  endRampUp =  103L;                      // We don't want to Ramp Up as we are supposed to be moving
  sDelay = startDelay;
  steps = rampSteps +100L; 
}

void setSteps(long sSteps, int sWait)
{
int oldSpeed;

  rampSteps = (long)(((float)sSteps / 2.0));      // the number of on/off cycles 
  startDelay =   HzToTimeDelay(globals.minStepperHz);          
  targetDelay =  HzToTimeDelay(globals.maxStepperHz);  
  if(((startDelay-targetDelay) * globals.microSteps) < rampSteps) // Don't go faster than the max stepper speed
    rampSteps = ((startDelay-targetDelay) * globals.microSteps);  
  startRampDown = rampSteps;           // Steps to start ramping down - steps count down so we start at a high #
  endRampUp = (sSteps - rampSteps);    // Steps to stop ramping up -- steps count down so we start at a high #
  oldSpeed = sDelay;                   // save the old speed
  sDelay = startDelay;                 // set starting delay
  Timer1.initialize(sDelay);           // set the interrupt timer
  stepcount = 0L;                      // Zero the step counter
  steps = sSteps;                      // GO for it
  if(sWait){
    while(steps){
      ;                                  // Wait until done
    }
  }
  sDelay = oldSpeed; 
}

void addDevice(char *pPrompt)
{
  const char noDevSig[] PROGMEM = NO_DEVICE_SIG;  // clear the signature
  Form addDeviceForm(lcd);
  BoolField aConfirmField(addDeviceForm, pPrompt, yStr, nStr, false);
  int event  = LCD_BUTTON_NONE;
  lcd.clear(); 
  addDeviceForm.defaultField();
  addDeviceForm.show();
  while(event != LCD_BUTTON_SELECT){
    event = lcd.getButton();
    if (addDeviceForm.dispatch(event) == FORM_CHANGED) {
        if (addDeviceForm.isCurrent(aConfirmField)) {
      }
    }
  }
  if (aConfirmField.value()){
    int eOffset = sizeof(configuration) + (sizeof(globals) * configuration.numDevices); 
    if((eOffset + sizeof(globals))< 2048){                    // Is there enough room in the EEPROM
      configuration.currDevice = configuration.numDevices + 1; // Increment the device we are using
      configuration.numDevices++;                              // Increment the Number of devices
      eOffset += sizeof(globals);
      EEPROM_writeAnything(0, configuration);                  // then update configuration in EEPROM 
      EEPROM_writeAnything(eOffset, globals);                  // then write new device to EEPROM 
      wdt_enable(WDTO_15MS);                          // REBOOT - turn on the WatchDog and don't stroke it.
      for(;;) { 
        // do nothing and wait for the eventual... REBOOT
      }
    }
  }
}

void eraseEeprom(char *sPrompt)
{
  const char noDevSig[] PROGMEM = NO_DEVICE_SIG;  // clear the signature
  Form eraseForm(lcd);
  BoolField ConfirmField(eraseForm, sPrompt, yStr, nStr, false);
  int event  = LCD_BUTTON_NONE;
  lcd.clear(); 
  eraseForm.defaultField();
  eraseForm.show();
  while(event != LCD_BUTTON_SELECT){
    event = lcd.getButton();
    if (eraseForm.dispatch(event) == FORM_CHANGED) {
      if (eraseForm.isCurrent(ConfirmField)) {
      }
    }
  } 
  if (ConfirmField.value()){
    EEPROM_writeAnything(0, noDevSig);            // then write it to EEPROM we think there is no data there
    wdt_enable(WDTO_15MS);                          // REBOOT - turn on the WatchDog and don't stroke it.
    for(;;) { 
      // do nothing and wait for the eventual... REBOOT
    }
  }   
}
void loadDevice(char *lPrompt)
{
  Form loadForm(lcd);
  IntField devField(loadForm,"Device" , 0, (configuration.numDevices), 1, configuration.currDevice, "");
  int event  = LCD_BUTTON_NONE;
  lcd.clear(); 
  loadForm.defaultField();
  loadForm.show();
  //ConfirmField.setValue(0);
  while(event != LCD_BUTTON_SELECT){
    event = lcd.getButton();
    if (loadForm.dispatch(event) == FORM_CHANGED) {
      if (loadForm.isCurrent(devField)) {
  
      }
    }
  }
  if (devField.value() != configuration.currDevice){ // We changed the device
   configuration.currDevice = devField.value();     // So set the current device
   EEPROM_writeAnything(0, configuration);          // then write it to EEPROM we think there is no data there
   wdt_enable(WDTO_15MS);                           // REBOOT - turn on the WatchDog and don't stroke it.
   for(;;) { 
     // do nothing and wait for the eventual... REBOOT
   }
  }  
}

void doConfigMenu(void)
{
  int event = LCD_BUTTON_NONE;
  Form configForm(lcd);
  ListField goSpeedField(configForm,"<Set Speed>" ,TopMenu);
  ListField goJogStepField(configForm,"<Jog Step>" ,TopMenu);
  ListField goSetupField(configForm,"<Setup>" ,TopMenu);
  ListField goDeviceField(configForm,"<Devices>",TopMenu);


  lcd.clear(); 
  configForm.defaultField();
  configForm.show();
  while(event != LCD_BUTTON_SELECT){
    event = lcd.getButton();
    if (configForm.dispatch(event) == FORM_CHANGED) {
      if (configForm.isCurrent(goSpeedField)) {
        goSetSpeed();
        configForm.setCurrentField(&goSpeedField);
      }    
      if (configForm.isCurrent(goJogStepField)) {
        goSetJogStep();
        configForm.setCurrentField(&goJogStepField);
      }
      if (configForm.isCurrent(goSetupField)) {
        doSetupMenu();
        configForm.setCurrentField(&goSetupField);
      }
      if (configForm.isCurrent(goDeviceField)) {
        goDeviceMenu();
        configForm.setCurrentField(&goDeviceField);
      }
    }
  } 
}

void goDeviceMenu(void)
{
   
  Form deviceForm(lcd);
  BoolField cNewDevField(deviceForm, "New dev", yStr, nStr, false);
  BoolField cLoadDevField(deviceForm, "Load dev", yStr, nStr, false);
  BoolField cEraseDevField(deviceForm, "Erase all?", yStr, nStr, false);

  int event = LCD_BUTTON_NONE;

  lcd.clear();
  deviceForm.defaultField();
  deviceForm.show();
  while(event != LCD_BUTTON_SELECT ){
    event = lcd.getButton();
    if (deviceForm.dispatch(event) == FORM_CHANGED) {
      if (deviceForm.isCurrent(cNewDevField)) {
        // New Device, create new globals,init it and use it
        if (cNewDevField.value()){
          addDevice("NEW Dev?");
          deviceForm.setCurrentField(&cNewDevField);
          cNewDevField.setValue(false);
        }
      }
      if (deviceForm.isCurrent(cLoadDevField)) {
        // Load Device and save it
          loadDevice("LOAD Dev?");

        deviceForm.setCurrentField(&cLoadDevField);
      }
      if (deviceForm.isCurrent(cEraseDevField)) {
        // Erase Devicesand stat again
        if (cEraseDevField.value()){
          eraseEeprom("ERASE All?");
          cEraseDevField.setValue(false);
          deviceForm.setCurrentField(&cEraseDevField);            
        }  
      }
    }
  }
}  

void setdeviceType(int dType)
{
   switch(dType){
     case DEV_ROTARY:
        mainForm.removeField(&goTravelField_mm);
        break;  
     case DEV_LINEAR:
       mainForm.removeField(&goAngleField);
       mainForm.removeField(&goDivideField);
       mainForm.removeField(&goConTurnField);

       break;  
  }     
}

String formatNumStr(long sNum)
{
  String numstr(sNum);
  int slen = numstr.length();
  String frac;
  String whol;
  if(slen <3){
    switch(slen){
        case 1:
            numstr = numstr +"00";
            break;
        case 2:
            numstr = numstr +"0";
            break;
    }
    numstr = "0." + numstr;
  }
  else{
    frac = numstr.substring(slen -3);
    whol = numstr.substring(0,slen -3);
    numstr =whol + "." + frac;
  }  
 return numstr;  
}
void printDistanceVals(long DistToTravel, int step)
{
  const char mmStr[] PROGMEM = "mm";
  const char stpStr[] PROGMEM = "Step:";
  const char posStr[] PROGMEM = "Pos :";

  String dStr = formatNumStr(DistToTravel);
  
  String mStr = formatNumStr(DistToTravel * (long) step);
  lcd.clear();
  lcd.setCursor(0,0);
  lcd.print(stpStr);
  lcd.print(step);
  lcd.print(" "); 
  lcd.print(dStr); 
  lcd.setCursor(0,1);
  lcd.print(posStr); 
  lcd.print(mStr); 
  lcd.print(mmStr); 
}

void doTravel(long idistance)
{ 
   int event = LCD_BUTTON_NONE;
   long tmpSteps = (long)((((float)idistance/1000.0 / (float)globals.TravelDistRev) * (float)globals.stepsPerRev)*1000.0);
   long moveCount = 0;
   printDistanceVals(idistance,moveCount);
   while(event != LCD_BUTTON_SELECT){
      event = lcd.getButton();
      if(event != LCD_BUTTON_NONE){
        switch(event){
          case LCD_BUTTON_LEFT:
            moveCount--;
            digitalWrite( globals.DirPin, ANTICLOCKWISE );
            setSteps(tmpSteps * 2, 1);
            printDistanceVals(idistance,moveCount);
            break;
          case LCD_BUTTON_RIGHT:
            moveCount++;
            digitalWrite( globals.DirPin, CLOCKWISE );
            setSteps(tmpSteps * 2, 1);
            printDistanceVals(idistance,moveCount);
            break;
        } 
      }
  } 
}

void enterDistance(void)
{
  Form distanceForm(lcd);
  GetMicroField distField(distanceForm,"Dist" , 9999999, GETINTFIELD_READ_WRITE);
  int event = LCD_BUTTON_NONE;
  distField.setValue(distance);
  distanceForm.defaultField();
  distanceForm.show();
  while(event != LCD_BUTTON_SELECT){
    event = lcd.getButton();
    if (distanceForm.dispatch(event) == FORM_CHANGED) {
        if (distanceForm.isCurrent(distField)) {
          distance = distField.value();        
       }
    }
  }
  if(distance)
    doTravel(distance);
  distanceForm.removeField(&distField); 
}
  

/// --------------------------
/// Custom ISR Timer Routine
/// --------------------------
void timerIsr()
{
   // Toggle Stepper
   if(steps){
      state = state ^ 1;
      digitalWrite( globals.StepPin, state );
      if(!state){
           if(steps >= endRampUp){
             stepcount++; stepcount++;
             stepmod = (long)stepcount % (long)globals.microSteps;
             if(stepmod == 0){
               sDelay--;
             Timer1.initialize(sDelay);     // set the interrupt timer 1 step slower (eg. increase the timer delay)
             }
          }
          else if(steps <= startRampDown){
             stepcount--; stepcount--;
            stepmod = ((long)stepcount % (long)globals.microSteps);
            if(stepmod == 0){
              sDelay++;
              Timer1.initialize(sDelay);     // set the interrupt timer 1 step slower (eg. increase the timer delay)
          }
        } 
      }      
      steps--;
      if(!goinghome){
        if(state){
          if(digitalRead( globals.DirPin) == CLOCKWISE ){
          homePos++;
          if(globals.devType == DEV_ROTARY){                  // Updated  so as not to reset on a full revolution if a linear device
            if(homePos > (long)globals.stepsPerRev)
              homePos = 0L;      
          }
        } 
        else{
          homePos--;
          if(globals.devType == DEV_ROTARY){
            if(homePos < 0L)   
              homePos =  (long)globals.stepsPerRev;
          }
        }
      }   
    }
  } 
}

