
/*
This example demonstrates how to use the Form and Field classes from the
LCD library to provide a simple UI on the 16x2 LCD display.

This example is placed into the public domain.
*/

// include the library code:
#include <LCD.h> 
#include <Form.h>
#include <TextField.h>
#include <TimeField.h>
#include <DegreeField.h>
#include <IntField.h>
#include <BoolField.h>
#include <ListField.h>
#include <GetIntField.h>
// Initialize the LCD
LCD lcd;

// Note: if you are using the USBDroid and have reassigned pin D9 on the LCD shield to some
// other pin (e.g. A1), then you will need to initialize the shield with something like:
// LCD lcd(A1);
// See also: http://www.freetronics.com/pages/combining-the-lcd-keypad-shield-and-the-usbdroid


const char item_Eggs[] PROGMEM = "Eggs";
const char item_Cheese[] PROGMEM = "Cheese";
const char item_Pumpkin[] PROGMEM = "Pumpkin";
 ListItem const ingredients[] PROGMEM = {
    item_Eggs,
    item_Cheese,
    item_Pumpkin,
    0
 };
// Create the main form and its fields.
Form mainForm(lcd);
TextField welcomeField(mainForm, "Form example", "v1.0");
TimeField timeField(mainForm, "Time since reset", 24, TIMEFIELD_READ_ONLY);
DegreeField degreeField(mainForm, "Rotation", 360, DEGREEFIELD_READ_WRITE);
IntField volumeField(mainForm, "Volume", 0, 100, 5, 85, "%");
BoolField ledField(mainForm, "Status LED", "On", "Off", true);
ListField ingredient(mainForm, "  <Main Menu>  ", ingredients);
GetIntField stepField(mainForm, "Number of steps", 9999, GETINTFIELD_READ_WRITE);

#define STATUS_LED 13

void setup() {
  // Status LED initially on.
  pinMode(STATUS_LED, OUTPUT);
  digitalWrite(STATUS_LED, HIGH);
  
  // Enable the screen saver, which will automatically blank the screen after 10 seconds.
  // The screen will wake up again when a button is pressed or lcd.display() is called.
  lcd.enableScreenSaver();
  
  // Show the main form for the first time.
  mainForm.show();
}

void loop() {
  // Update the number of seconds since reset:
  timeField.setValue(millis() / 1000);

  // Dispatch button events to the main form.
  int event = lcd.getButton();
  if (mainForm.dispatch(event) == FORM_CHANGED) {
    if (mainForm.isCurrent(ledField)) {
      if (ledField.value())
        digitalWrite(STATUS_LED, HIGH);
      else
        digitalWrite(STATUS_LED, LOW);
    }  
  }
}

